/** \file srvip.h
 *
 * server-side (and client-side) structs, macros, prototypes, etc.
 *
 */
#ifndef SRVIP_H_
#define SRVIP_H_

#include "project.def"

#include "srvcore.h"

#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#if defined(DOS) /* DOS */
#include <sys/socket.h>
#elif defined(UNIX) /* UNIX */
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <stddef.h>
#elif defined(NIOS) /* NIOS */
#include "socket.h"
#endif 
extern int gUseLoopback;
extern int gNoLoopback;
#include "ttycmd.h"

#ifndef NumTCPChan
# define NumTCPChan	5
#endif
#ifndef UDP_PORT
# define UDP_PORT 9003
#endif
extern int gtUDPPort;
#ifndef TCP_PORT
# define TCP_PORT 8503
#endif
extern int gtTCPPort;
#ifndef GCAST_PORT
# define GCAST_PORT 8004
#endif
extern int gtGCastPort;
#ifndef NETSRV_PORT
# define NETSRV_PORT 9002
#endif
extern int gtNetSrvPort;
#ifndef MCAST_PORT
# define MCAST_PORT 9503
#endif
extern int gtMCastPort;
#ifndef STREAM_PORT
# define STREAM_PORT 10003
#endif
extern int gtStrmPort;
#ifndef PIPE_PORT
# define PIPE_PORT 10503
#endif
extern int gtPipePort;
#ifndef UDP_CLN_PORT
# define UDP_CLN_PORT 8050
#endif
extern int gtUDPClnPort;
#ifndef UDP_CLN_RANGE
# define UDP_CLN_RANGE (UDP_CLN_PORT+1000)
#endif
#ifndef NETBCST_ADDR
# define NETBCST_ADDR "0.0.0.0"
#endif
extern char *gtNetLclBcast;
#ifndef NETSRV_ADDR
# define NETSRV_ADDR "238.1.1.2"
#endif
extern char *gtNetSrvAddr;
#ifndef MCAST_ADDR
# define MCAST_ADDR "238.1.1.1"
#endif
extern char *gtMCastAddr;
#ifndef GCAST_ADDR
# define GCAST_ADDR "238.1.1.0"
#endif
extern char *gtGCastAddr;
#ifndef MCAST_TTL
# define MCAST_TTL 16
#endif
extern int gtMCastTTL;
#ifndef IPC_PATH
# define IPC_PATH "/tmp/"
#endif
#define IPC_PERM S_IRWXU

#ifndef MAX_TCPCONNECTIONS
# define MAX_TCPCONNECTIONS 32
#endif

#ifndef MAX_IPCCONNECTIONS
# define MAX_IPCCONNECTIONS 64
#endif

#ifndef MAX_DBGCONNECTIONS
# define MAX_DBGCONNECTIONS 5
#endif

#ifndef DEFAULT_MAX_TCPMSG_SIZE
# define DEFAULT_MAX_TCPMSG_SIZE 100000000
#endif

#define MIN_FDSETS 16

#define MTU 1460

#ifndef MIN_RCVBUF
/**
 * \internal
 * \brief Default minimal number of receive buffers for socket options.
 */
# define MIN_RCVBUF 0x1000
#endif

#ifndef MAX_RCVBUF
/**
 * \internal
 * \brief Default maximal number of receive buffers for socket options.
 */
# define MAX_RCVBUF 0x10000
#endif

#ifndef MIN_SNDBUF
/**
 * \internal
 * \brief Default minimal number of send buffers for socket options.
 *
 */
# define MIN_SNDBUF MIN_RCVBUF
#endif

#ifndef MAX_SNDBUF
/**
 * \internal
 * \brief Default maximal number of send buffers for socket options.
 *
 */
# define MAX_SNDBUF (MAX_RCVBUF / 2)
#endif

#ifndef DEFAULT_TCP_CONNECTION_TIMEOUT
# define DEFAULT_TCP_CONNECTION_TIMEOUT 200
#endif

#ifndef TIME_SERVER_HOST
# define TIME_SERVER_HOST NULL
#endif

#define LOOPBACKADDR "127.0.0.1"

#ifndef IPPORT_ECHO
# define IPPORT_ECHO 7
#endif

#define NETSRV_SOCKET 6565

#define MY_ADDRESS_FILE "myaddr.csv"

#define MCAST_ADDRESS_FILE "mcastaddr.csv"
#define MCAST_MASK_FILE "mcastmask.csv"
#define GCAST_ADDRESS_FILE "gcastaddr.csv"
#define GCAST_MASK_FILE "gcastmask.csv"

#define NETS_ACCESS  0
#define NETS_NETWORK 1
extern char *netsListFile[2];

typedef struct tcpIdleConnectionStruct
{
  int idx; /**< FEC table index */
  SOCKET sck; /**< TCP socket */
  int trp; /**< transport (TCP or STREAM) */
  int ttl; /**< time to live */
  struct tcpIdleConnectionStruct *prv;
  struct tcpIdleConnectionStruct *nxt;
} IDLE_CONNECTION;
extern IDLE_CONNECTION *tcpIdleConnectionList;
IDLE_CONNECTION *GetIdleConnection(int idx,int inetProtocol);
IDLE_CONNECTION *AddIdleConnection(SOCKET sck,int idx,int inetProtocol);
IDLE_CONNECTION *RemoveIdleConnection(IDLE_CONNECTION *ic);
IDLE_CONNECTION *GetIdleConnectionBySocket(SOCKET sck,int inetProtocol);

/**
 * \internal
 * \brief stream parameters used for TCP Stream transport
 *
 */
typedef struct
{
  ClnHdr  *dst;
  BYTE *payload;
} StreamParams;
int putStreamParams(ClnHdr *c,BYTE *payload);

typedef struct 
{
  ClnHdr dst;
  BYTE payload[TCP_BUFFER_SIZE];
  int msgsize;
  int isready;
  int isbusy;
} StreamDataTable;
typedef struct tcpbckt
{
  int idx;                  /* FEC index             */
  SOCKET sck;               /* TCP connection socket */
  int ptr;                  /* bucket pointer        */
  int siz;                  /* bucket target size    */
  int len;                  /* bucket buffer length  */
  int clrlen;               /* collector buffer length */
  BYTE *buf;                /* bucket buffer         */
  BYTE *clrbuf;             /* collector buffer      */
# ifdef MULTITHREADED
  THRHANDLE rcvThr;         /* recvThread handle     */
  THRHANDLE sndThr;         /* sendThread handle     */
  SEMHANDLE sndSem;         /* sendThread semaphore  */
  int sdTblHead;            /* stream data table head */
  int sdTblTail;            /* stream data table tail */
  int sdTblSize;            /* stream data table size */
  StreamDataTable *sdTbl;   /* stream data table (queue) */
# endif
  struct tcpbckt *nxt;      /* this is a linked list */
} TCPBCKT;
extern TCPBCKT *bcktList;
TCPBCKT *findBucket(SOCKET sck,int lock);
TCPBCKT *getBucket(SOCKET sck,int buffer_length,int collector_length,int type,int source);
void freeBucket(SOCKET sck);

typedef struct 
{
  SOCKET sck;
  int type;
} TCPSCKT;
extern TCPSCKT *tcpSckTbl;
extern int nTCPsck;
extern int maxFdSets;
extern int tcpConTblSize;
extern int gMaxTcpMsgSize;

extern TCPSCKT *dbgSckTbl;
extern int nDBGsck;
extern int dbgConTblSize;
extern int gAllowRemoteDebugging;

extern int ipxSrvSck;
extern int ipxClnSck;
extern int ipxMcastClnSck;
extern int ipcSrvSck;
#define soerr(s) {cc = udp_socket_error; strcat(txt,s); goto err;}
#define addIpSubStr(b,s) if ((s).sin_zero[0] != 0) sprintf(&(b)[strlen(b)],"/%d",(s).sin_zero[0])

extern struct sockaddr_in gIncomingIPfec;
extern IPXAddress gIncomingIPXfec;

extern struct in_addr tMCastAddr;
extern struct in_addr tMCastGenAddr;
extern struct in_addr tGCastAddr;
extern struct in_addr tGCastGenAddr;
extern struct in_addr tNetSrvAddr;
extern struct sockaddr_in tMCastAddrSck;
extern struct sockaddr_in tGCastAddrSck;
extern struct sockaddr_in tNetSrvAddrSck;
extern fd_set ext_set;
extern int externalFdSets;
extern struct sockaddr_in *IPBcastList;
extern int NrOfIPBcastNets;
void FD_MERGE(fd_set fdsrc,fd_set *fddst);
void addServerSocketsToSet(fd_set *fdset);
void addClientSocketsToSet(fd_set *fdset);
extern void (*socketAcceptEntryFcn)(void);
void CloseConnectionSocket(SOCKET sck,int inetProtocol);
void closeIPConnectionToFec(int index,SOCKET sck);
int ListenForIPMulticasts(void);
int ListenForIPNetGlobals(void);
UINT32 getmyipaddr(void);
int getSrvIPAddr(void);
int getIPControlNets(const char *filename,int type,int *nr,struct sockaddr_in **list);
int addRegNetsLst(const char *filename,int type,int *nr,struct sockaddr_in **list,NAME32 *ipaddr,int addsiz);
int rmvRegNetsLst(const char *filename,int type,int *nr,struct sockaddr_in **list,NAME32 *ipaddr,int addsiz);
int attachPortToMulticastGroup(SOCKET sck,unsigned short port,struct in_addr grpAddr,char *grpType);
int detachPortFromMulticastGroup(SOCKET sck,unsigned short port,struct in_addr grpAddr,char *grpType);
void freeBucketList(void);

TINE_EXPORT int SetSrvUDPPort(int port);
TINE_EXPORT int GetSrvUDPPort(void);
TINE_EXPORT int SetSrvTCPPort(int port);
TINE_EXPORT int GetSrvTCPPort(void);
TINE_EXPORT int SetGCastPort(int port);
TINE_EXPORT int GetGCastPort(void);
TINE_EXPORT int SetMCastPort(int port);
TINE_EXPORT int GetMCastPort(void);
TINE_EXPORT int OpenIpcPipe(char *pipeName);
TINE_EXPORT int openIPCPipe(char *pipeName);
TINE_EXPORT int CloseIpcPipe(char *pipeName);
TINE_EXPORT int GetIpcPipeName(char *pipeName);

/* SetClnUDPPort() */
/**
 * \brief Sets the TINE client UDP base port
 * 
 * TINE attempts to obtain the first free port beginning UDP client base port. 
 * If for some reason the client base port needs to be adjusted away from its default
 * value of 8050, then this routine can be used.
 *
 * \param port is the base UDP port to use for the TINE clients
 *  
 * \return 0 on success or a TINE error code
 */
TINE_EXPORT int SetClnUDPPort(int port);

/* GetClnUDPPort() */
/**
 * \brief Gets the TINE client UDP base port
 * 
 * \return The client base port
 */
TINE_EXPORT int GetClnUDPPort(void);

/* SetMCastTTL() */
/**
 * \brief Sets the TINE multicast Time-To-Live
 * 
 * If the multicast Time-To-Live parameter (number of 'hops') needs to be
 * adjusted away from its default value of 16 then this routine can be used.
 *
 * \param ttl is the time-to-live value to use for multicasts
 *  
 * \return 0 on success or a TINE error code
 */
TINE_EXPORT int SetMCastTTL(int ttl);

/* GetMCastTTL() */
/**
 * \brief Gets the TINE multicast Time-To-Live
 * 
 * \return The multicast Time-To-Live
 */
TINE_EXPORT int GetMCastTTL(void);

/* SetPacketMTU() */
/**
 * \brief Sets the TINE Packet MTU for the UDP server socket
 * 
 * The TINE default transport protocol is UDP and the default UDP datagram
 * MTU (maximum transport unit) is 1472 bytes (i.e. the ethernet packet MTU).
 * TINE will supply its own packet reassembly for data transfers larger than this.
 * Many modern operating systems have network stacks that will provide 
 * UDP packet reassembly up to 64 Kbytes. This generally a much more efficent
 * packet reassembly as it happens at the driver level.  If a higher performance
 * for very large data transfer is required, this routine can be called at the
 * server prior to any call to SystemInit().  
 *
 * \param mtu is the desired maximum transport unit for the server socket. 
 * The default is 1472 bytes.  The minimum accepted value is 512 and the 
 * maximum accepted value is 64000 bytes.
 *  
 * \return 0 on success or a TINE error code
 *
 * \note Not all operating systems are able to handle MTUs greater than the standard
 * value of 1472 (notably the standard VxWorks build, and some legacy operating systems).
 * Clients on these operating systems will not be able to receive large data transfers
 * from servers sending at greater MTUs!
 */
TINE_EXPORT int SetPacketMTU(int mtu);

/* GetPacketMTU() */
/**
 * \brief Gets the TINE UDP server packet MTU
 * 
 * \return The UDP server packet MTU
 */
TINE_EXPORT int GetPacketMTU(void);
TINE_EXPORT int SetNetSrvAddr(char *addr);
TINE_EXPORT char *GetNetSrvAddr(void);
TINE_EXPORT int SetMCastAddr(char *addr);
TINE_EXPORT char *GetMCastAddr(void);
TINE_EXPORT int SetGCastAddr(char *addr);
TINE_EXPORT char *GetGCastAddr(void);

/* SetMaxTcpMessageSize() */
/**
 * \brief Sets the TINE server maximum STREAM message segment size
 * 
 * When using CM_STREAM to acquire data from a server, very large amounts of
 * data can be passed to the stream socket.  In general the network stack will not
 * be able to prcess an indeterminatnly large amount and the TINE transport system
 * will need to parcel the transmission into segments.  The 
 * maximum tcp message size determines the size of the underlying parcel.
 * By default, this is 100000000 bytes.
 * This value can be adjusted by calling this routine at the server prior to any 
 * call to SystemInit().  
 *
 * \param value is the desired maximum STREAM message segment size
 * The default is 100000000 bytes.  The minimum accepted value is 1472 byte, There
 * is no maximum, however the network stack may refuse to trasmit if the value is too 
 * large.
 *  
 * \return 0 on success or a TINE error code
 *
 * \note this applies only to trasmissions in CM_STREAM mode, which is in turn only
 * applicable to multi-threaded builds of the TINE libraries.
 */
TINE_EXPORT void SetMaxTcpMessageSize(int value);

/* GetMaxTcpMessageSize() */
/**
 * \brief Gets the TINE server maximum STREAM message segment size
 * 
 * \return The the TINE server maximum STREAM message segment size
 */
TINE_EXPORT int GetMaxTcpMessageSize(void);

/* _SystemAssignBufferSpace() */
/**
 * \brief Assigns specific buffer capacity for certain configurable TINE sockets.
 * 
 * The standard TINE client and server sockets (both UDP, TCP, and STREAM) are initiated with a 
 * reasonable default amount of receive (64 Kbyte) and send (32 Kbyte) buffer capacity.  If for any
 * performance reason, more or less buffer capactiy is needed, this routine should be called prior to
 * any calls to SystemInit() or SystemCycle().  For example, VxWorks builds which do not have a large 
 * amount of network resources might want to set these values smaller than the defaults.  Video system
 * clients expecting extreme amounts of incoming data might want to set these values to a much larger
 * amount
 *
 * \param rcvBufferSpace is the amount of receive buffer space (in bytes) to assign to the configurable TINE sockets
 * \param sndBufferSpace is the amount of send buffer space (in bytes) to assign to the configurable TINE sockets
 *  
 * \return 0 on success or a TINE error code
 */
TINE_EXPORT int _SystemAssignBufferSpace(UINT32 rcvBufferSpace,UINT32 sndBufferSpace);
#define SystemAssignBufferSpace(a,b) _SystemAssignBufferSpace(a,b)

/* SetUseLoopback() */
/**
 * \fn TINE_EXPORT void SetUseLoopback(int value)
 * \brief Sets the 'useLoopBack' flag to the value given (boolean).
 * 
 * If the useLoopBack flag is 'TRUE' then client-side calls to a server on the same host as
 * the client will use the network socket layer as the communication avenue between client and 
 * server, as opposed to shared memory (windows) or pipes (unix).
 *
 * \param value should be non-zero (TRUE) to turn this flag on or FALSE to turn it off.  FALSE is
 * the default.
 *
 * \see GetUseLoopback().
 */
TINE_EXPORT void SetUseLoopback(int value);
/* GetUseLoopback() */
/**
 * \fn TINE_EXPORT int GetUseLoopback(void)
 * \brief Returns the current value of the 'useLoopBack' flag.
 * 
 * If the useLoopBack flag is 'TRUE' then client-side calls to a server on the same host as
 * the client will use the network socket layer as the communication avenue between client and 
 * server, as opposed to shared memory (windows) or pipes (unix).
 *
 * \return the current setting of the useLoopBack flag.
 *
 * \see SetUseLoopback().
 */
TINE_EXPORT int GetUseLoopback(void);
/* SetForceSharedMemory() */
/**
 * \fn TINE_EXPORT void SetForceSharedMemory(int value)
 * \brief Disables or enables the 'useLoopBack' flag according to the value given (boolean).
 * 
 * If the useLoopBack flag is 'FALSE' (default) then client-side calls to a server on the same host as
 * the client will attempt to use shared memory (windows) or pipes (unix) as a more expedient 
 * communications avenue between client and server.  If the server however appears not to support
 * such communications (either a legacy server or a java server, for example), then the loopback address
 * is used as a fallback.  If the server is known a priori to support such alternate communication
 * channels, then this 'fallback' strategy can be turned off by making use of this call.
 *
 * \param value should be non-zero (TRUE) to turn this feature on or FALSE to turn it off.  FALSE is
 * the default.
 *
 * \see GetForceSharedMemory().
 */
TINE_EXPORT void SetForceSharedMemory(int value);
/* GetForceSharedMemory() */
/**
 * \fn TINE_EXPORT int GetForceSharedMemory(void)
 * \brief returns the current setting of this feature.
 * 
 * If the useLoopBack flag is 'FALSE' (default) then client-side calls to a server on the same host as
 * the client will attempt to use shared memory (windows) or pipes (unix) as a more expedient 
 * communications avenue between client and server.  If the server however appears not to support
 * such communications (either a legacy server or a java server, for example), then the loopback address
 * is used as a fallback.  If the server is known a priori to support such alternate communication
 * channels, then this 'fallback' strategy can be turned off by making use of SetForceSharedMemory().
 * The current setting can be seen by using this call.
 *
 * \return the current setting of the forcedSharedMemory feature.
 *
 * \see SetForceSharedMemory().
 */
TINE_EXPORT int GetForceSharedMemory(void);

/**
 * \fn TINE_EXPORT void SetTcpConnectionTableSize(int value)
 * \brief sets the tcp connection table size.
 *
 * Transport communication goes via UDP by default.  However by specifying
 * either CM_CONNECT or CM_STREAM a caller can request connected stream
 * transport via TCP.  In this case a slot is reserved in a tcp connection
 * managment table for the resulting communications socket.  This is a 
 * table which is NOT allowed to grow in size but is restricted to the 
 * size seen at initialization (default = 32 entries).  If it is known
 * a priori that TCP communication will be used in a big way, then this
 * table size should be adjusted prior to system initialization 
 * (i.e. a call to SystemInit()).
 *
 * \param value is the desired tcp connection table size (minimum 8)
 *
 * \see GetTcpConnectionTableSize().
 */
TINE_EXPORT void SetTcpConnectionTableSize(int value);

/**
 * \fn TINE_EXPORT int GetTcpConnectionTableSize(void)
 * \brief returns the current setting of the tcp connection table size.
 * 
 * Transport communication goes via UDP by default.  However by specifying
 * either CM_CONNECT or CM_STREAM a caller can request connected stream
 * transport via TCP.  In this case a slot is reserved in a tcp connection
 * managment table for the resulting communications socket.  This is a 
 * table which is NOT allowed to grow in size but is restricted to the 
 * size seen at initialization (default = 32 entries).  
 *
 * \return the current setting of the tcp connection table size.
 *
 * \see SetTcpConnectionTableSize().
 */
TINE_EXPORT int GetTcpConnectionTableSize(void);

/**
 * \fn TINE_EXPORT void SetAllowRemoteDebugging(int value)
 * \brief enables/disables remote debugging
 *
 * The 'attachfec' tool is usually used on the same host as
 * the running server, in which case the debugging output is 
 * forwarding via an efficient local named pipe.  
 * In some cases, it might be desired to make use of the 'attachfec' on a remote
 * host, in which case debugging output is transfered via a stream
 * connection (less efficient).  This feature can be turned OFF by
 * making use of this API call and passing a value of FALSE prior
 * to a call to SystemInit().
 *
 * \param value is the desired setting (default = TRUE).
 *
 * \see GetAllowRemoteDebugging().
 */
TINE_EXPORT void SetAllowRemoteDebugging(int value);

/**
 * \fn TINE_EXPORT int GetAllowRemoteDebugging(void)
 * \brief returns the current setting of this value
 * 
 * The 'attachfec' tool is usually used on the same host as
 * the running server, in which case the debugging output is 
 * forwarding via an efficient local named pipe.  
 * In some cases, it might be desired to make use of the 'attachfec' on a remote
 * host, in which case debugging output is transfered via a stream
 * connection (less efficient).  Whether this feature is available is
 * determined by this setting.
 *
 * \return the current setting (default = TRUE).
 *
 * \see SetAllowRemoteDebugging().
 */
TINE_EXPORT int GetAllowRemoteDebugging(void);

# if !defined(VXWORKS) && !defined(VMS) && !defined(NIOS) && !defined(DOS) && !defined(SMALL_TINE_LIB)
void initAddrPortsFromCsv(void);
void initAddrPortsFromEnv(void);
extern int gInitAddrPortsDone;
#endif

extern int gListenForGlobals;
extern int gWaitingOnSelect;
#if defined(VXWORKS) && (_WRS_VXWORKS_MAJOR < 6)
struct hostent *gethostbyname(const char *hstname);
struct hostent *gethostbyaddr(const char *hstaddr, int hstlen, int hsttype);
#endif
#ifdef WIN32
int GetErrno(void);
int WinsockLoaded(void);
#endif
#ifdef UNIX
typedef struct
{
  int sck;
  int fecIdx;
} ClnPipeSck;
# define PIPE_TBL_CAPACITY 100
extern ClnPipeSck gClnPipeSckTbl[PIPE_TBL_CAPACITY];
extern int nClnPipeSck;
int addClnPipeSck(int sck,int idx);
int rmvClnPipeSck(int sck);
typedef struct 
{
  SOCKET sck;
  int type;
  char name[16];
  struct sockaddr_in ipAddr;
} IPCSCKT;
extern char gIpcTinePipe[64];
extern int nIPCsck;
extern IPCSCKT *ipcSckTbl;
extern int ipcConTblSize;
int isUnixSocket(int sck);
void ClosePipeSocket(int sck);
# define writeto(fd,bf,sz,fg,ad,ln) (\
  !isUnixSocket(fd) ? sendto(fd,bf,sz,fg,ad,ln) : write(fd,bf,sz)\
)
# define readfrom(fd,bf,sz,fg,ad,ln) (\
  !isUnixSocket(fd) ? recvfrom(fd,bf,sz,fg,ad,(socklen_t *)ln) : read(fd,bf,sz)\
)
# define getpeeraddr(fd,ad,ln) getpeername(fd,ad,(socklen_t *)ln)
#elif defined(VXWORKS)
# define IsConnProt(p) ((p) == TCP || (p) == STREAM)
# define writeto(fd,bf,sz,fg,ad,ln) (IsConnProt(inetProtocol) ? send(fd,bf,sz,fg) : sendto(fd,bf,sz,fg,ad,ln))
# define readfrom(fd,bf,sz,fg,ad,ln) recvfrom(fd,bf,sz,fg,ad,ln)
# define getpeeraddr(fd,ad,ln) getpeername(fd,ad,(int *)ln)
#else
# define writeto(fd,bf,sz,fg,ad,ln) sendto(fd,bf,sz,fg,ad,ln)
# define readfrom(fd,bf,sz,fg,ad,ln) recvfrom(fd,bf,sz,fg,ad,ln)
# define getpeeraddr(fd,ad,ln) getpeername(fd,ad,(int *)ln)
#endif

int processTCPDataStream(TCPBCKT **pbckt,int nread,int type);
void recvIncomingIPDataErr(SOCKET sck,int prot,int nread);
int processTCPDataPacket(SOCKET sck,BYTE *bf,int nread);
#define ProcessDataPacket(sck,bf) {\
  if (IncomingNetworkProtocol & (PIPE|TCP|STREAM)) {\
    n = processTCPDataPacket(sck,bf,nread);\
  } else {\
    if (n == nread) consumeData(0,bf);\
    else if (tineDebug) dbglog("%s: input error %d bytes read instead of %d",fecprot,nread,n);\
  }\
}

#define recvIncomingIPData(sck) {\
  if (FD_ISSET(sck,&fdset)) {\
    if ((nread=readfrom(sck,(char *)clnWorkArea,MIN_WORK_SIZE,0,(struct sockaddr *)&gIncomingIPfec,&len)) > 0)\
    {\
      if (IncomingNetworkProtocol == TCP) {\
        len = sizeof(gIncomingIPfec);\
        getpeeraddr(sck,(struct sockaddr *)&gIncomingIPfec,&len);\
        inet_ntoa_b(gIncomingIPfec.sin_addr,gFecAddrBuffer);\
      } else if (IncomingNetworkProtocol == PIPE) {\
        strncpy(gFecAddrBuffer,"local pipe",16);\
      } else {\
        inet_ntoa_b(gIncomingIPfec.sin_addr,gFecAddrBuffer);\
        n = (int)PeekWord(*(UINT16 *)clnWorkArea);\
      }\
      bufptr = clnWorkArea; ProcessDataPacket(sck,bufptr);\
    } else if (soerrno != EWOULDBLOCK) {\
      if (IncomingNetworkProtocol & (PIPE|TCP|STREAM)) {\
        recvIncomingIPDataErr(sck,IncomingNetworkProtocol,nread);\
        sck = 0;\
      }\
    }\
    if (tineDebug > 3) {\
      if (nread <= 0) dbglog("%s: read error",fecprot);\
      else dbglog("%s: %d bytes read from %d",fecprot,nread,n);\
    }\
  }\
}

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* SRVIP_H_ */
